function [x, flag, n_func] = optimize( op, mu, x_start )
% OPPTIMIZE optimizes  the problem op using exterior
% penalty method, penalty parameter mu and initial point
% x_start. Return values are x -- final iteration,
% flag - termination flag (> 0 if ok), n_func -- number of
% functional evaluations

tol = 1e-6;
maxit = 400;

%construct the penalty function first
%here we use simple quadratic penalty
obj_str = sprintf( '(%s)', op.obj_func );
grd_str = sprintf( '(%s)', op.obj_grad );
Gs_str  = '[';
for i = 1:op.m
  obj_str = sprintf( '%s - %0.10g * log(-(%s))', ...
		     obj_str, mu, op.con_func{i} );
  grd_str = sprintf( '%s - %0.10g * 1/(%s)*(%s)', ...
		     grd_str, mu, op.con_func{i}, op.con_grad{i} );
	    
  Gs_str = sprintf( '%s (%s);', ...
		     Gs_str, op.con_func{i} );
	    
end

Gs_str = sprintf( '%s ]', Gs_str );

x = conjgrad( ...
    x_start, ...
    inline( obj_str, 'x' ), ...
    inline( grd_str, 'x' ), ...
    inline( Gs_str, 'x' ), ...
    tol, ...
    maxit ...
    );

n_func = 0;
flag = 0;

